@@ -1,8 +1,11 @@
+import json
 import logging
 
-from octue.exceptions import ProtectedAttributeException
+from octue.definitions import OUTPUT_STRANDS
 from octue.mixins import Identifiable, Loggable, Serialisable, Taggable
 from octue.resources.manifest import Manifest
+from octue.utils.encoders import OctueJSONEncoder
+from octue.utils.folders import get_file_name_from_strand
 from twined import ALL_STRANDS, Twine
 
 
@@ -44,7 +47,7 @@ class Analysis(Identifiable, Loggable, Serialisable, Taggable):
     :parameter logger: Optional logging.Logger instance attached to the analysis
     """
 
-    def __init__(self, twine, **kwargs):
+    def __init__(self, twine, skip_checks=False, **kwargs):
         """ Constructor of Analysis instance
         """
 
@@ -53,27 +56,50 @@ class Analysis(Identifiable, Loggable, Serialisable, Taggable):
             twine = Twine(source=twine)
 
         self.twine = twine
+        self._skip_checks = skip_checks
 
         # Pop any possible strand data sources before init superclasses (and tie them to protected attributes)
-        strand_kwargs = dict((name, kwargs.pop(name, None)) for name in ALL_STRANDS)
-        for strand_name, strand_data in strand_kwargs.items():
-            self.__setattr__(f"_{strand_name}", strand_data)
+        strand_kwargs = ((name, kwargs.pop(name, None)) for name in ALL_STRANDS)
+        for strand_name, strand_data in strand_kwargs:
+            setattr(self, f"{strand_name}", strand_data)
 
         # Init superclasses
         super().__init__(**kwargs)
 
-    def __setattr__(self, name, value):
-        """ Override setters for protected attributes (the strand contents may change, but the strands themselves
-        shouldn't be changed after instantiation)
-        """
-        if name in ALL_STRANDS:
-            raise ProtectedAttributeException(f"You cannot set {name} on an instantiated Analysis")
+    def finalise(self, output_dir=None):
+        """ Validates and serialises output_values and output_manifest, optionally writing them to files
 
-        super().__setattr__(name, value)
+        If output_dir is given, then the serialised outputs are also written to files in the output directory
 
-    def __getattr__(self, name):
-        """ Override public getters to point to protected attributes (the strand contents may change, but the strands
-        themselves shouldn't be changed after instantiation)
+        :parameter output_dir: path-like pointing to directory where the outputs should be saved to file (if None, files
+         are not written)
+        :type output_dir:  path-like
+
+        :return: dictionary of serialised strings for values and manifest data.
+        :rtype: dict
         """
-        if name in ALL_STRANDS:
-            return getattr(self, f"_{name}")
+
+        # Using twined's validate_strand method gives us sugar to check for both extra outputs
+        # (e.g. output_values where there shouldn't be any) and missing outputs (e.g. output_values is None when it
+        # should be a dict of data)
+        serialised = dict()
+        for k in OUTPUT_STRANDS:
+            self.logger.debug(f"Serialising {k}")
+            att = getattr(self, k)
+            if att is not None:
+                att = json.dumps(att, cls=OctueJSONEncoder)
+
+            serialised[k] = att
+
+        self.logger.debug("Validating serialised output json against twine")
+        self.twine.validate(**serialised)
+
+        # Optionally write the serialised strand to disk
+        for k in OUTPUT_STRANDS:
+            if output_dir and serialised[k] is not None:
+                file_name = get_file_name_from_strand(k, output_dir)
+                self.logger.debug(f"Writing {k} to file {file_name}")
+                with open(file_name, "w") as fp:
+                    fp.write(serialised[k])
+
+        return serialised
